﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;
using SchedulingTool.Entities;
using WpfApplication1;

namespace SchedulingTool.ViewModels
{
    public class MySchedulingViewModel : INotifyPropertyChanged
    {
        #region Fields
        private readonly MyResourceList _resourceMembers;
        private RelayCommand<string> _addResourceCommand;
        private RelayCommand _loadResourcesCommand;
        private ObservableCollection<DateRange> _availableSlots = new ObservableCollection<DateRange>();
        #endregion

        #region Properties
        public MyResourceList ResourceMembers
        {
            get { return this._resourceMembers; }
        }

        public IEnumerable<MyAppointment> Appointments
        {                       
            get
            {
                return this.ResourceMembers.SelectMany(resourceMember => resourceMember.Appointments).ToList();
            }
        }

        public ObservableCollection<DateRange> AvailableTimeSlots
        {
            get { return this._availableSlots; }
            set { this.SetProperty(() => this.AvailableTimeSlots, ref this._availableSlots, value); }
        }
        #endregion

        #region Commands
        public RelayCommand LoadResourcesCommand
        {
            get { return this._loadResourcesCommand; }
            internal set
            {
                this._loadResourcesCommand = value;
                this.NotifyPropertyChanged(() => this.LoadResourcesCommand);
            }
        }

        public RelayCommand<string> AddResourceCommand
        {
            get { return this._addResourceCommand; }
            internal set
            {
                this._addResourceCommand = value;
                this.NotifyPropertyChanged(() => this.AddResourceCommand);
            }
        }
        #endregion

        #region Ctor
        public MySchedulingViewModel()
        {
            this._resourceMembers = new MyResourceList();
            this.AddResourceCommand = new RelayCommand<string>(this.AddResource, this.CanAddResource);
            this.LoadResourcesCommand = new RelayCommand(this.LoadResources, this.CanLoadResources);
            var slots = new ObservableCollection<DateRange>
                                           {
                                               new DateRange(DateTime.Now, DateTime.Now.AddMinutes(325)),
                                               new DateRange(DateTime.Now.AddMinutes(345), DateTime.Now.AddMinutes(370))
                                           };
            this.AvailableTimeSlots = slots;

            this.NotifyPropertyChanged(()=>this.AvailableTimeSlots);
        }
        #endregion

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;

        protected void NotifyPropertyChanged<TProperty>(Expression<Func<TProperty>> property)
        {
            if (this.PropertyChanged == null)
                return;

            var lambda = (LambdaExpression)property;
            var unaryExpression = lambda.Body as UnaryExpression;
            MemberExpression memberExpression = unaryExpression != null
                                                    ? (MemberExpression)unaryExpression.Operand
                                                    : (MemberExpression)lambda.Body;
            this.PropertyChanged(this, new PropertyChangedEventArgs(memberExpression.Member.Name));
        }

        protected void SetProperty<TProperty>(Expression<Func<TProperty>> property, ref TProperty field, TProperty value)
        {
            field = value;
            this.NotifyPropertyChanged(property);
        }
        #endregion

        #region Methods
        private bool CanLoadResources()
        {
            return true;
        }

        private void LoadResources()
        {
            this.ResourceMembers.Add(new MyResource {Id = "David", IsVisible = true, Name = "David"});
            this.ResourceMembers.Add(new MyResource {Id = "Kim", IsVisible = true, Name = "Kim"});
            this.ResourceMembers.Add(new MyResource {Id = "Harry", IsVisible = true, Name = "Harry"});
            foreach (MyResource resourceMember in this.ResourceMembers)
            {

                var appointment = new MyAppointment
                                      {
                                          Description = "Coffee Break",
                                          OwningResourceId = resourceMember.Id,
                                          OwningCalendarId = resourceMember.Calendar.Id,
                                          OriginalOccurrenceEnd = DateTime.Now.AddHours(1.5),
                                          OriginalOccurrenceStart = DateTime.Now,
                                          Start = DateTime.Now,
                                          End = DateTime.Now.AddHours(1.5),
                                          IsTimeZoneNeutral = true,
                                          Id = resourceMember.Id + DateTime.Now.Ticks,
                                          IsVisible = true,
                                          Subject = "Break",
                                          TaskType = "Animal Pickup",
                                          OwningCalendar = resourceMember.Calendar,
                                      };
                appointment.Locations.Add(new Location{Address="Scrum Room 1"});
                appointment.Locations.Add(new Location { Address = "Discussion Room 2" });
                appointment.Locations.Add(new Location { Address = "Discussion Room 3" });

                resourceMember.Appointments.Add(appointment);
            }
            this.NotifyPropertyChanged(()=>Appointments);
        }

        private bool CanAddResource(string name)
        {
            return this.ResourceMembers.FirstOrDefault(item => String.Compare(item.Id, name) == 0) == null;
        }

        private void AddResource(string name)
        {
            this.ResourceMembers.Add(new MyResource {Id = name, IsVisible = true, Name = name});
            this.NotifyPropertyChanged(() => Appointments);
        }
        #endregion
    }
}